// SPDX-License-Identifier: Apache-2.0 OR MIT
// SPDX-FileCopyrightText: 2025 Russ Fellows <russ.fellows@gmail.com>

pub mod npz;
#[cfg(feature = "hdf5")]
pub mod hdf5;
pub mod tfrecord;
pub mod raw;

pub use npz::{array_to_npy_bytes, build_npz, build_multi_npz, read_npy_bytes, read_npz_array, list_npz_arrays};
#[cfg(feature = "hdf5")]
pub use hdf5::build_hdf5;
pub use tfrecord::{build_tfrecord, build_tfrecord_with_index, TfRecordWithIndex};
pub use raw::build_raw;

